//=============================================================================
// Project:    SLP - An Interpreter for Super Logic Programs / Static Semantics
// Module:     scope.h
// Purpose:    Set of all variables in the current formula
// Last Change:04.04.2002
// Language:   C++
// Author:     Stefan Brass
// Email:      Stefan.Brass@informatik.uni-giessen.de, sbrass@sis.pitt.edu
// Address:    Universitaet Giessen, Arndtstr. 2, 35392 Giessen, Germany
// Copyright:  (c) 2000-2002 by Stefan Brass
// Licence:    Distributed under the conditions of the GNU Public Licence.
//=============================================================================

//=============================================================================
// Include File Frame:
//=============================================================================

#ifndef	SCOPE_INCLUDED
#define SCOPE_INCLUDED

//=============================================================================
// Used Types and Macros:
//=============================================================================

#ifndef VER_INCLUDED
#include "ver.h"
#endif

#ifndef BOOL_INCLUDED
#include "bool.h"
#endif

#ifndef STR_INCLUDED
#include "str.h"
#endif

#ifndef CHECK_INCLUDED
#include "check.h"
#endif

#ifndef VAR_INCLUDED
#include "var.h"
#endif

//=============================================================================
// All variables in the current formula (class without instances):
//=============================================================================

class scope_c {

	public:

//-----------------------------------------------------------------------------
// Class Methods:
//-----------------------------------------------------------------------------

	// Reset (make variable list empty):
	static inline void reset(void)
		{ NumVars = 0;
		  NextVar = 0;}

	// Find/Construct a Variable:
	static var_t find(str_t name_ptr, str_t name_end);


	// Return first variable:
	static inline var_t first(void)
		{ CHECK(valid(), "scope_c::first: Internal inconsistency");
		  if(NumVars > 0) {
			NextVar = 1;
			return(Vars[0]);
		  }
		  else
			return(0);
		}

	// Return next variable:
	static inline var_t next(void)
		{ CHECK(valid(), "scope_c::next: Internal inconsistency");
		  if(NumVars > NextVar)
			return(Vars[NextVar++]);
		  else
			return(0);
		}

//-----------------------------------------------------------------------------
// Debugging Support:
//-----------------------------------------------------------------------------

#ifdef	VER_DEBUG
	public:

	// Integrity Check:
	static bool_t	valid(void);

	// Print variable list:
	static void print(void);
#endif

//-----------------------------------------------------------------------------
// Private Members:
//-----------------------------------------------------------------------------

	private:

	// List of variables:
	static var_t	Vars[VER_MAX_VARS];

	// Number of variables in the list:
	static int	NumVars;

	// Index of next variable in the loop/scan over all variables:
	static int	NextVar;

//-----------------------------------------------------------------------------
// End of Class:
//-----------------------------------------------------------------------------

};

//=============================================================================
// End of Include File:
//=============================================================================

#endif

